--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: FILE
-- @author    Valentin Schmidt
-- @version   0.2
--****************************************************************************

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

  -- me.ancestor = xtra("fileIO").new()
  
  -- me.$.include MOVIESCRIPT
  -- me.$.include(libdir&"file_fileio.ls")
  
  return me
end

--------------------------------------
-- Opens file with specified mode, returns file pointer (xtra instance)
-- @param {string} tFile - MySQL dump SQL string
-- @param {string} tMode - r, rb, w, wb, a, ab, rw, rwb, wr, wrb
-- @return {instance|false}
--------------------------------------
on fopen (me, tFile, tMode)
  fp = xtra("fileIO").new()
  
  case (tMode) of
    "r","rb": fp.openFile(tFile, 1)
    "w","wb":
      --if not fileExists(tFile) then
      
      fp.openFile(tFile, 2)
      err = fp.status()
      if (err=0) then fp.delete()
      --else if (err and not (err = -37)) then return false
    
      fp.createFile(tFile)
      fp.openFile(tFile, 2)
    "rw","wr","rwb","wrb": fp.openFile(tFile, 0)
    "a","ab":
      fp.openFile(tFile, 2)
      fp.setPosition(fp.getLength())
    otherwise:
      return 0
  end case
  
  if (fp.status()<>0) then fp = 0
  return fp
end

--------------------------------------
-- Closes open file
-- @param {instance} fp
--------------------------------------
on fclose (me, fp)
  fp.closeFile()
  fp = 0
end

----------------------------------------
-- Deletes open file
-- @param {instance} fp
-- @return {bool}
----------------------------------------
on fdelete (me, fp)
  err = fp.delete()
  return (err=0)
end

----------------------------------------
-- Reads n bytes from open file, returns bytearray
-- @param {instance} fp
-- @param {integer} n
-- @return {bytearray}
----------------------------------------
on freadbytes(me, fp, n)
  ba = fp.readByteArray(n)
  ba.position = 1
  return ba
end
 
----------------------------------------
-- Writes byteArray to open file
-- @param {instance} fp
-- @param {bytearray} ba
----------------------------------------
on fwritebytes (me, fp, ba)
  return fp.writeByteArray(ba)
end

----------------------------------------
-- Reads n bytes from open file, returns string
-- @param {instance} fp
-- @param {integer} n
-- @return {string}
----------------------------------------
on freadstring(me, fp, n)
  ba = freadbytes(fp, n)
  ba.position = 1
  return ba.readRawString(ba.length)
end
 
----------------------------------------
-- Writes string to open file
-- @param {instance} fp
-- @param {string} str
----------------------------------------
on fwritestring (me, fp, str)
  fp.writeString(str)
end

--------------------------------------
-- Returns position in open file
-- @param {instance} fp
-- @return {integer}
--------------------------------------
on ftell (me, fp)
  return fp.getPosition()
end

--------------------------------------
-- Sets position in open file
-- whence: either integer or symbol
-- 0 or #SEEK_SET: set position to offset bytes. (default)
-- 1 or #SEEK_CUR: set position to current position plus offset.
-- 2 or #SEEK_END: set position to end of file plus offset.
-- @param {instance} fp
-- @param {integer} pos
-- @param {integer|symbol} [whence=0]
-- @return {bool}
--------------------------------------
on fseek (me, fp, pos, whence)
  if voidP(whence) then whence = 0
  else if ilk(whence)=#symbol then
    whence = [#SEEK_SET, #SEEK_CUR, #SEEK_END].getPos(whence)
    if whence>0 then whence=whence-1
  end if
  
  case (whence) of
    0: fp.setPosition(pos)  
    1: fp.setPosition(fp.getPosition() + pos) 
    2: fp.setPosition(fp.getLength() + pos) 
  end case
  
  return (fp.status()=0)
end

--------------------------------------
-- Returns size of open file
-- @param {instance} fp
-- @return {integer}
--------------------------------------
on fsize (me, fp)
  return fp.getLength()
end

--**************************************
-- global functions, no file pointer passed
--**************************************

----------------------------------------
-- Returns file size
-- @param {string} tFile
-- @return {integer}
----------------------------------------
on size (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if err then return err
  len = fp.getLength()
  fp.closeFile()
  fp = 0
  return len
end

----------------------------------------
-- Truncates file
-- @param {string} tFile
-- @param {integer} tSize
-- @return {bool}
----------------------------------------
on truncate (me, tFile, tSize)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  data = fp.readByteArray(tSize)
  fp.delete()
  fp.createFile(tFile)
  fp.openFile(tFile, 2)
  fp.writeByteArray(data)
  err = fp.status()
  fp.closeFile()
  fp=0
  return (err=0)
end

----------------------------------------
-- Deletes file
-- @param {string} tFile
-- @return {bool}
----------------------------------------
on delete (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 0)
  err = fp.status()
  if not (err) then err = fp.delete()
  fp = 0
  return (err=0)
end

----------------------------------------
-- is_file() ?
-- @param {string} tFile
-- @return {bool}
----------------------------------------
on exists (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  stat = fp.status()
  fp.closeFile()
  fp = 0
  -- 0: OK: item is a file
  -- -36: I/O Error: item is a folder
  -- -37: Bad File Name: does not exist
  return (stat=0)
end


----------------------------------------
-- Reads whole file, returns string
-- @param {string} tFile
-- @return {string|false}
----------------------------------------
on getString (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if (err) then return false
  ret = fp.readFile()
  fp.closeFile()
  fp = 0
  return ret
end

----------------------------------------
-- Saves string as file
-- @param {string} tFile
-- @param {string} tString
-- @return {bool}
----------------------------------------
on putString (me, tFile, tString)
  fp = xtra("fileIO").new()  
  fp.openFile(tFile, 2)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return false
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return false
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return false
  fp.writeString(tString)
  fp.closeFile()
  fp=0
  return true
end

----------------------------------------
-- Returns file as ByteArray
-- @param {string} tFile
-- @return {byteArray|false}
----------------------------------------
on getBytes (me, tFile)
  fp = xtra("fileIO").new()
  fp.openFile(tFile, 1)
  err = fp.status()
  if (err) then return false
  data = fp.readByteArray(fp.getLength())
  fp.closeFile()
  fp = 0
  return data
end

----------------------------------------
-- Saves ByteArray to file
-- @param {string} tFile
-- @param {byteArray} tString
-- @return {bool}
----------------------------------------
on putBytes (me, tFile, tByteArray)
  fp = xtra("fileIO").new()  
  fp.openFile(tFile, 2)
  err = fp.status()
  if not (err) then fp.delete()
  else if (err and not (err = -37)) then return false
  fp.createFile(tFile)
  err = fp.status()
  if (err) then return false
  fp.openFile(tFile, 2)
  err = fp.status()
  if (err) then return false
  fp.writeByteArray(tByteArray)
  fp.closeFile()
  fp=0
  return true
end

----------------------------------------
-- Displays open file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
on selectFileOpen (me, tTitle, filterMask, defaultFileName)  
  fio = xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn = fio.displayOpen()
  fio = 0
  return string(fn)
end

----------------------------------------
-- Displays save file dialog
-- @param {string} tTitle
-- @param {string} filterMask
-- @param {string} defaultFileName
-- @return {string}
----------------------------------------
on selectFileSave (me, tTitle, filterMask, defaultFileName)  
  fio = xtra("fileIO").new()  
  fio.setFilterMask(filterMask)
  fn = fio.displaySave(tTitle, defaultFileName)
  fio = 0
  return string(fn)
end

----------------------------------------
-- Select folder via save file dialog
-- @param {string} tTitle
-- @return {string}
----------------------------------------
on selectFolder (me, tTitle)
  if voidP(tTitle) then tTitle = "Select Folder"
  fio = new(xtra "fileIO")
  fn = fio.displaySave(tTitle, "*.*")
  err = fio.status()
  fio = 0
  if string(fn)=EMPTY then return ""
  if err<>0 then
    put "Save File Error:" && fio.error(err)
    --exit
    return ""
  end if
  pd = the last char of _movie.path
  od = _player.itemDelimiter
  _player.itemDelimiter = pd
  delete the last item of fn
  _player.itemDelimiter = od
  put pd after fn
  return fn
end

----------------------------------------
-- @static
-- @param {string} str
-- @return {propList}
----------------------------------------
--on parseFilename (me, str)
--  res = [:]
--  
--  if str contains "/" then pd = "/"
--  else if str contains "\" then pd = "\"
--  else pd = ":"
--  	
--  -- basename
--  if the last char of str = pd then res[#basename] = ""
--  else
--  	parts = me.explode(pd, str)
--		res[#basename] = parts[parts.count]
--		delete char str.length-res[#basename].length to str.length of str
--	end if
--  
--  -- drive
--  if (pd="\" OR pd="/") AND str.char[2]=":" then
--  	res[#drive] = str.char[1]
--  	delete char 1 to 2 of str
--  end if
--  
--  -- path
--  res[#path] = str
--   
--  return res
--end
